#Python 3.9.9
#numpy==1.20.1
#matplotlib==3.3.4

import numpy as np
import matplotlib.pyplot as plt

def simulation(length, delta_length, binary_image1):# input: length:        filament length used for simulation
    angle_counter = [0 for x in range(360)]         #        delta_length:  step size after which 'filament == inside' is controlled 
    heatmap1 = binary_image1                        #        binary_image1: size 512x512, inside == 1, outside == 0 
    print('filament length [px]:', length)
    for i in range(0, 512):                         # loop through rows
        if (i % 50) == 0:
            print('row ', i)
        for j in range(0, 512):                     # loop through columns
            counter = 0
            if binary_image1[i][j] != 0:
                for phi in range(0, 360):
                    k = 0                           # check for multiple points on the filament if it is inside, controlled by delta_length
                    inside = True                   # -> concave shapes can be used for binary image
                    while k < length:
                        x = i + int(np.sin(phi * 2 * np.pi / 360) * k)
                        y = j + int(np.cos(phi * 2 * np.pi / 360) * k)
                        if binary_image1[x][y] == 0:
                            inside = False
                            break
                        k += delta_length
                    if inside:                      # if filament inside: save the angle
                        counter += 1
                        angle_counter[int(phi)] += 1
                        k = 0
                        while k < length:
                            x = i + int(np.sin(phi * 2 * np.pi / 360) * k)
                            y = j + int(np.cos(phi * 2 * np.pi / 360) * k)
                            heatmap1[x][y] += 1
                            k += delta_length
    plt.imshow(heatmap1)
    plt.colorbar(orientation = "vertical")
    plt.savefig('simulation_result_image_circle.png', format = 'png' ,dpi=1200)
    plt.show(block = True)
    
    angle = np.linspace(0, 359, 360)
    plt.plot(angle, angle_counter)
    plt.xlabel('angle')
    plt.ylabel('counts')
    plt.ylim(0,None)
    plt.show(block=True)

# upload binary image
filename = "binary_image_circle.txt"
binary_image = np.loadtxt(filename, delimiter = ",")
plt.imshow(binary_image)
plt.colorbar(orientation = "vertical")
plt.savefig('binary_image_circle.png', format = 'png' ,dpi=1200)

plt.show(block = True)

# start simulation - here: filament length = 20px
filament_length = 20 #units: px
delta = 5 # resólution in which length is tested,5 is a good starting point and used for all our simulations
simulation(filament_length, delta, binary_image)