#Script written by Andreas Fink and Charlotte R. Doll
#using the following modules: 
#imageio==2.9.0
#matplotlib==3.3.4
#numpy==1.20.1
#scikit_learn==1.1.1
#Python 3.9.9

import numpy as np
import matplotlib.pyplot as plt
import imageio
from sklearn.decomposition import PCA
import os
from datetime import datetime

#reading in the soax output and Image files
image_file = "GUV_actin_with_gaussian_blur.tif"
soax_output = "GUV_actin_SOAX_output.txt"

def distance_2d(p1, p2):
	return (np.sqrt((p1[0]-p2[0])**2 + (p1[1] - p2[1])**2)) 

image = imageio.imread(image_file)
f_in = open(soax_output)
snakes = []
snakes_x = []
snakes_y = []
    
#read in data from the SOAX output - test for your own system!
snake_counter = -1 #gives number of snakes starting from 0
for line in f_in.readlines()[28:]:
    if "[" in line:
        continue

    if '#' in line:
        snakes.append([])
        snakes_x.append([])
        snakes_y.append([])

        snake_counter +=1
        continue

    if snake_counter != -1:
        line = line.split()
        if line != []:
            snakes_x[snake_counter].append(float(line[2]))
            snakes_y[snake_counter].append(float(line[3]))
            
for i in range(len(snakes_x)):
    plt.plot(snakes_x[i], snakes_y[i], lw=1, color='magenta')
    
plt.imshow(image, cmap = plt.cm.gray)
plt.title('Snakes from SOAX output (magenta)')
plt.savefig('example_guv_after_soax.png', format = 'png' ,dpi=1200) 
plt.show()

#do principal component analysis of Actin filaments to simplify further analysis
length_after_pca = []
res =[]
pca_x = []
pca_y = []
for i in range(len(snakes_x)):
    length_after_pca.append(0)
    pca = PCA(n_components=1)
    pca.fit(np.asarray([snakes_x[i], snakes_y[i]]).T)
    X_pca = pca.transform(np.asarray([snakes_x[i], snakes_y[i]]).T)
    X_new = pca.inverse_transform(X_pca)
    plt.plot(X_new[:,0], X_new[:,1], color="green", lw=1)
    plt.plot(snakes_x[i], snakes_y[i], lw=0.7, color="magenta") 
    pca_x.append(X_new[:,0])
    pca_y.append(X_new[:,1])
    res.append(X_new)
    plt.axis('equal')
        
    #compute length of filament after projection on first PC
    length_after_pca[i] += distance_2d([X_new[0][0], X_new[0][1]],[X_new[-1][0], X_new[-1][1]]) #length in px
    
#plot the result
plt.imshow(image, cmap="gray")
plt.title('after principal component analysis (green)')
plt.savefig('example_guv_after_PCA.png', format = 'png' ,dpi=1200)
plt.show(block=True)

#Calculate the angle of the filament wrg. to the y-axis. plot the result. 
#You should save the list iof you want to do further plotting of more GUVs together.
angle = []
for i in range(len(snakes_x)):
    angle.append(np.abs(np.arcsin((pca_x[i][0]-pca_x[i][-1])/length_after_pca[i]))*180/np.pi)
plt.xlabel("Angle in Degree")
plt.ylabel("counts")
plt.hist(angle, bins=20)
plt.show(block=True)
